package com.winhc.task;

import com.alibaba.excel.EasyExcel;
import com.alibaba.fastjson.JSONObject;
import com.aliyun.odps.Column;
import com.winhc.task.bean.Summary;
import com.winhc.task.common.Constant;
import com.winhc.task.listener.NoModelDataListener;
import com.winhc.task.util.BaseUtils;
import com.winhc.task.util.EasyExcelUtil;
import com.winhc.task.util.OdpsUtils;
import com.winhc.task.util.QueryBuilderUtils;
import lombok.val;
import org.apache.commons.lang3.StringUtils;
import org.apache.hadoop.hbase.TableName;
import org.apache.hadoop.hbase.client.Get;
import org.elasticsearch.action.search.SearchRequest;
import org.elasticsearch.action.search.SearchResponse;
import org.elasticsearch.client.RestHighLevelClient;
import org.elasticsearch.index.query.WrapperQueryBuilder;
import org.elasticsearch.search.SearchHit;
import org.elasticsearch.search.builder.SearchSourceBuilder;
import org.junit.jupiter.api.Test;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Qualifier;
import org.springframework.boot.test.context.SpringBootTest;

import java.io.IOException;
import java.nio.charset.StandardCharsets;
import java.util.*;
import java.util.concurrent.CompletableFuture;
import java.util.stream.Collectors;
import java.util.stream.Stream;

import static com.winhc.task.util.BaseUtils.cleanup;
import static com.winhc.task.util.BaseUtils.toDBC;

/**
 * @author π
 * @Description:
 * @date 2023/5/30 9:15
 */
@SpringBootTest
public class CompanyHumanJob {

    @Autowired
    @Qualifier(value = "v6")
    RestHighLevelClient getClient;

    @Autowired
    @Qualifier(value = "v5")
    RestHighLevelClient getOldClient;

    @Test
    public void start1() throws Exception {

        String fileName = "C:\\Users\\batmr\\Desktop\\yhc\\人企测试样本（3144条）(1).xlsx";
        // 这里 只要，然后读取第一个sheet
        List<Map<Integer, String>> dataList = new ArrayList<>();
        EasyExcel.read(fileName, new NoModelDataListener(dataList)).sheet().doRead();
        System.out.println(dataList.size());

        val futures = new ArrayList<CompletableFuture<JSONObject>>();

        dataList.stream().map(m -> {
            JSONObject j = new JSONObject();
            String company_name = m.get(0);
            String human_name = m.get(1);
            j.fluentPut("company_name", company_name)
                    .fluentPut("human_name", human_name);
            return j;
        }).forEach(d -> {
            futures.add(fetchEs(d));
        });

        CompletableFuture.allOf(futures.toArray(new CompletableFuture[0])).get();
        List<JSONObject> re = futures.stream().map(x -> {
            try {
                return x.get();
            } catch (Exception e) {
                throw new RuntimeException("error 1111 :" + e.getMessage(), e);
            }
        }).filter(Objects::nonNull).collect(Collectors.toList());

        val futures2 = new ArrayList<CompletableFuture<JSONObject>>();

        re.forEach(t -> {
            futures2.add(fetchOldEs(t));
        });

        CompletableFuture.allOf(futures2.toArray(new CompletableFuture[0])).get();
        List<JSONObject> re2 = futures2.stream().map(x -> {
            try {
                return x.get();
            } catch (Exception e) {
                throw new RuntimeException("error 1111 :" + e.getMessage(), e);
            }
        }).filter(Objects::nonNull).collect(Collectors.toList());

        saveExcel(re2);
    }

    @Test
    public void start2() throws Exception {

        String fileName = "C:\\Users\\batmr\\Desktop\\yhc\\人企测试样本_company_id.xlsx";
        // 这里 只要，然后读取第一个sheet
        List<Map<Integer, String>> dataList = new ArrayList<>();
        EasyExcel.read(fileName, new NoModelDataListener(dataList)).sheet().doRead();
        System.out.println(dataList.size());

        //判断人员类型
        val futures = new ArrayList<CompletableFuture<List<JSONObject>>>();
        dataList.stream().map(m -> {
            JSONObject j = new JSONObject();
            String company_name = m.get(0);
            String human_name = m.get(1);
            String company_id = m.get(2);
            String human_pid = m.get(3);
            j.fluentPut("company_name", company_name)
                    .fluentPut("human_name", human_name)
                    .fluentPut("company_id", company_id)
                    .fluentPut("human_pid", human_pid)
            ;
            return j;
        }).forEach(d -> {
            try {
                futures.add(fetchAllCompany(d));
            } catch (IOException e) {
                e.printStackTrace();
            }
        });


        CompletableFuture.allOf(futures.toArray(new CompletableFuture[0])).get();
        List<JSONObject> re = futures.stream().flatMap(x -> {
            try {
                return x.get().stream();
            } catch (Exception e) {
                throw new RuntimeException("error 1111 :" + e.getMessage(), e);
            }
        }).filter(Objects::nonNull).collect(Collectors.toList());

        val futures2 = new ArrayList<CompletableFuture<JSONObject>>();

        re.forEach(t -> {
            futures2.add(fetchEsById(t));
        });

        CompletableFuture.allOf(futures2.toArray(new CompletableFuture[0])).get();
        List<JSONObject> re2 = futures2.stream().map(x -> {
            try {
                return x.get();
            } catch (Exception e) {
                throw new RuntimeException("error 1111 :" + e.getMessage(), e);
            }
        }).filter(Objects::nonNull).collect(Collectors.toList());

        saveExcelV2(re2);
    }

    private CompletableFuture<JSONObject> fetchEsById(JSONObject data) {
        return CompletableFuture.supplyAsync(() -> {
            try {
                String sub_company_id = data.getString("sub_company_id");
                String sub_company_name = queryCompanyNameByCompanyId(sub_company_id);
                data.fluentPut("sub_company_name", sub_company_name);
                return data;
            } catch (Exception e) {
                e.printStackTrace();
                data.fluentPut("sub_company_name", "");
                return data;
            }
        }, BaseUtils.COMMON_POOL);

    }

    private CompletableFuture<List<JSONObject>> fetchAllCompany(JSONObject data) throws IOException {

        return CompletableFuture.supplyAsync(() -> {
            try {
                String human_pid = data.getString("human_pid");
                if (StringUtils.isBlank(human_pid)) return new ArrayList<>();
                String tmp_data = JSONObject.toJSONString(data);
                Map<String, Set<String>> map = queryAllCompany(human_pid);
                List<JSONObject> re = map.entrySet().stream().map(m -> {
                    String company_id = m.getKey();
                    Set<String> value = m.getValue();
                    JSONObject tj = JSONObject.parseObject(tmp_data);
                    String labels = String.join("|", value);
                    tj.fluentPut("label", labels);
                    tj.fluentPut("sub_company_id", company_id);
                    return tj;
                }).collect(Collectors.toList());

                return re;
            } catch (Exception e) {
                e.printStackTrace();
                return new ArrayList<>();
            }
        }, BaseUtils.COMMON_POOL);
    }

    private CompletableFuture<JSONObject> fetchEs(JSONObject data) {
        return CompletableFuture.supplyAsync(() -> {
            try {
                String company_name = data.getString("company_name");
                String human_name = data.getString("human_name");
                if ("3M中国有限公司".equals(company_name)) {
                    company_name = toDBC(company_name);
                }
                String company_id = queryCompanyId(company_name, human_name);
                data.fluentPut("company_id", company_id);
                return data;
            } catch (Exception e) {
                e.printStackTrace();
                data.fluentPut("company_id", "");
                return data;
            }
        }, BaseUtils.COMMON_POOL);

    }

    private CompletableFuture<JSONObject> fetchOldEs(JSONObject data) {
        return CompletableFuture.supplyAsync(() -> {
            try {
                String company_id = data.getString("company_id");
                if (StringUtils.isBlank(company_id)) return data;
                String human_name = data.getString("human_name");
                String human_pid = queryPid(company_id, human_name);
                data.fluentPut("human_pid", human_pid);
                return data;
            } catch (Exception e) {
                e.printStackTrace();
                data.fluentPut("human_pid", "");
                return data;
            }
        }, BaseUtils.COMMON_POOL);

    }

    private void saveExcel(List<JSONObject> re) {
        List<List<String>> dataList = re.stream().map(d -> {
            List<String> dataRow = new ArrayList<>();
            dataRow.add(d.getString("company_name"));
            dataRow.add(d.getString("human_name"));
            dataRow.add(d.getString("company_id"));
            dataRow.add(d.getString("human_pid"));
            return dataRow;
        }).collect(Collectors.toList());

        EasyExcelUtil easyExcelUtil = new EasyExcelUtil();

        String fileName = "C:\\Users\\batmr\\Desktop\\yhc\\人企测试样本_company_id.xlsx";

        List<List<String>> head = Stream.of("企业名称", "法定代表人", "company_id", "human_pid")
                .map(Collections::singletonList)
                .collect(Collectors.toList());
        easyExcelUtil.init(fileName, "sheet1", head);
        easyExcelUtil.doExportExcel(dataList);
        //关闭流
        easyExcelUtil.finish();
    }

    private void saveExcelV2(List<JSONObject> re) {
        List<List<String>> dataList = re.stream().map(d -> {
            List<String> dataRow = new ArrayList<>();
            dataRow.add(d.getString("company_name"));
            dataRow.add(d.getString("human_name"));
            dataRow.add(d.getString("company_id"));
            dataRow.add(d.getString("human_pid"));
            dataRow.add(d.getString("sub_company_id"));
            dataRow.add(d.getString("sub_company_name"));
            dataRow.add(d.getString("label"));
            return dataRow;
        }).collect(Collectors.toList());

        EasyExcelUtil easyExcelUtil = new EasyExcelUtil();

        String fileName = "C:\\Users\\batmr\\Desktop\\yhc\\人企测试样本_label.xlsx";

        List<List<String>> head = Stream.of("企业名称", "法定代表人", "company_id", "human_pid", "sub_company_id", "sub_company_name", "label")
                .map(Collections::singletonList)
                .collect(Collectors.toList());
        easyExcelUtil.init(fileName, "sheet1", head);
        easyExcelUtil.doExportExcel(dataList);
        //关闭流
        easyExcelUtil.finish();
    }

    private Map<String, Set<String>> queryAllCompany(String pid) throws IOException {
        if (StringUtils.isBlank(pid)) {
            return new HashMap<>();
        }
        String dsl = "{\n" +
                "  \"query\": {\n" +
                "    \"bool\": {\n" +
                "      \"must\": [\n" +
                "        {\n" +
                "          \"terms\": {\n" +
                "            \"deleted\": [\n" +
                "              \"0\",\n" +
                "              \"1\"\n" +
                "            ]\n" +
                "          }\n" +
                "        },\n" +
                "        {\n" +
                "          \"bool\": {\n" +
                "            \"should\": [\n" +
                "              {\n" +
                "                \"term\": {\n" +
                "                  \"legal_entities.id\": {\n" +
                "                    \"value\": \"&condition\"\n" +
                "                  }\n" +
                "                }\n" +
                "              },\n" +
                "              {\n" +
                "                \"term\": {\n" +
                "                  \"holder_id\": {\n" +
                "                    \"value\": \"&condition\"\n" +
                "                  }\n" +
                "                }\n" +
                "              },\n" +
                "              {\n" +
                "                \"term\": {\n" +
                "                  \"hid\": {\n" +
                "                    \"value\": \"&condition\"\n" +
                "                  }\n" +
                "                }\n" +
                "              }\n" +
                "            ],\n" +
                "            \"minimum_should_match\": 1\n" +
                "          }\n" +
                "        }\n" +
                "      ]\n" +
                "    }\n" +
                "  }\n" +
                "}";

        String ds1 = dsl.replaceAll("&condition", pid);
        SearchRequest searchRequest = new SearchRequest("winhc_index_rt_company", "winhc_index_rt_company_holder", "winhc_index_rt_company_staff");
        SearchSourceBuilder searchSourceBuilder = new SearchSourceBuilder();
        searchSourceBuilder.size(10000);
        if (StringUtils.isNotEmpty(ds1)) {
            WrapperQueryBuilder query = QueryBuilderUtils.getQuery(JSONObject.parseObject(ds1));
            searchSourceBuilder.query(query);
        }
        searchRequest.source(searchSourceBuilder);
        SearchResponse searchResponse = getClient.search(searchRequest);
        SearchHit[] hits = searchResponse.getHits().getHits();

        Map<String, Set<String>> data_tmp = new HashMap<>();

        Arrays.stream(hits).forEach(m -> {
            String rowkey = m.getId();
            String index = m.getIndex();
            Map<String, Object> map = m.getSourceAsMap();
            String deleted = map.getOrDefault("deleted", "").toString();
            Set<String> set = new HashSet<>();

            String company_id;
            if (index.contains("company_holder")) {
                if (deleted.equals("0")) {
                    set.add("股东");
                } else {
                    set.add("历史股东");
                }
                company_id = map.getOrDefault("company_id", "").toString();

            } else if (index.contains("company_staff")) {
                if (deleted.equals("0")) {
                    set.add("高管");
                } else {
                    set.add("历史高管");
                }
                company_id = map.getOrDefault("company_id", "").toString();

            } else {
                if (deleted.equals("0")) {
                    set.add("法人");
                } else {
                    set.add("历史法人");
                }
                company_id = rowkey;
            }
            if (data_tmp.containsKey(company_id)) {
                set.addAll(data_tmp.get(company_id));
            }
            data_tmp.put(company_id, set);
        });

        return data_tmp;
    }

    private String queryCompanyId(String name, String legalName) throws IOException {

        String dsl = "{\n" +
                "  \"query\": {\n" +
                "    \"bool\": {\n" +
                "      \"must\": [\n" +
                "        {\n" +
                "          \"term\": {\n" +
                "            \"deleted\": {\n" +
                "              \"value\": \"0\"\n" +
                "            }\n" +
                "          }\n" +
                "        },\n" +
                "        {\n" +
                "          \"bool\": {\n" +
                "            \"should\": [\n" +
                "              {\n" +
                "                \"term\": {\n" +
                "                  \"cname.show.keyword\": {\n" +
                "                    \"value\": \"&condition\"\n" +
                "                  }\n" +
                "                }\n" +
                "              },\n" +
                "              {\n" +
                "                \"term\": {\n" +
                "                  \"history_name.show.keyword\": {\n" +
                "                    \"value\": \"&condition\"\n" +
                "                  }\n" +
                "                }\n" +
                "              }\n" +
                "            ],\n" +
                "            \"minimum_should_match\": 1\n" +
                "          }\n" +
                "        }\n" +
                "      ]\n" +
                "    }\n" +
                "  }\n" +
                "}";

        String ds1 = dsl.replaceAll("&condition", name);
        SearchRequest searchRequest = new SearchRequest("winhc_index_rt_company").types("company");
        SearchSourceBuilder searchSourceBuilder = new SearchSourceBuilder();
        searchSourceBuilder.size(10);
        if (StringUtils.isNotEmpty(ds1)) {
            WrapperQueryBuilder query = QueryBuilderUtils.getQuery(JSONObject.parseObject(ds1));
            searchSourceBuilder.query(query);
        }
        searchRequest.source(searchSourceBuilder);
        SearchResponse searchResponse = getClient.search(searchRequest);
        SearchHit[] hits = searchResponse.getHits().getHits();
        if (hits == null || hits.length == 0) {
            return "";
        } else if (hits.length == 1) {
            return hits[0].getId();
        } else {
            List<String> collect = Arrays.stream(hits).map(m -> {
                String id = m.getId();
                Map<String, Object> map = m.getSourceAsMap();
                Object legal_entities = map.getOrDefault("legal_entities", "");
                if (legal_entities == null) {
                    return null;
                }
                if (legal_entities.toString().contains(legalName)) {
                    return id;
                }
                return null;
            }).filter(Objects::nonNull).collect(Collectors.toList());
            if (collect.isEmpty()) return "";
            return collect.get(0);
        }
    }

    private String queryPid(String companyId, String human_name) throws IOException {
        String dsl = "{\n" +
                "  \"query\": {\n" +
                "    \"bool\": {\n" +
                "      \"must\": [\n" +
                "        {\n" +
                "          \"term\": {\n" +
                "            \"company_id\": {\n" +
                "              \"value\": \"&company_id\"\n" +
                "            }\n" +
                "          }\n" +
                "        },{\n" +
                "          \"term\": {\n" +
                "            \"human_name_cleanup\": {\n" +
                "              \"value\": \"&human_name\"\n" +
                "            }\n" +
                "          }\n" +
                "        }\n" +
                "      ]\n" +
                "    }\n" +
                "  }\n" +
                "}";

        String ds1 = dsl.replaceAll("&company_id", companyId).replaceAll("&human_name", cleanup(human_name));
        SearchRequest searchRequest = new SearchRequest("winhc_company_human_pid_mapping_v9").types("human");
        SearchSourceBuilder searchSourceBuilder = new SearchSourceBuilder();
        searchSourceBuilder.size(1);
        if (StringUtils.isNotEmpty(ds1)) {
            WrapperQueryBuilder query = QueryBuilderUtils.getQuery(JSONObject.parseObject(ds1));
            searchSourceBuilder.query(query);
        }
        searchRequest.source(searchSourceBuilder);
        SearchResponse searchResponse = getOldClient.search(searchRequest);
        List<String> collect = Arrays.stream(searchResponse.getHits().getHits())
                .map(x -> {
                    Map<String, Object> sourceAsMap = x.getSourceAsMap();
                    String human_pid = sourceAsMap.getOrDefault("human_pid", "").toString();
                    return human_pid;
                }).collect(Collectors.toList());
        if (collect.isEmpty()) return "";
        return collect.get(0);
    }

    private String queryCompanyNameByCompanyId(String companyId) throws IOException {
        String dsl = "{\n" +
                "  \"query\": {\n" +
                "    \"term\": {\n" +
                "      \"_id\": {\n" +
                "        \"value\": \"&company_id\"\n" +
                "      }\n" +
                "    }\n" +
                "  }\n" +
                "}";

        String ds1 = dsl.replaceAll("&company_id", companyId);
        SearchRequest searchRequest = new SearchRequest("winhc_index_rt_company").types("company");
        SearchSourceBuilder searchSourceBuilder = new SearchSourceBuilder();
        searchSourceBuilder.size(1);
        if (StringUtils.isNotEmpty(ds1)) {
            WrapperQueryBuilder query = QueryBuilderUtils.getQuery(JSONObject.parseObject(ds1));
            searchSourceBuilder.query(query);
        }
        searchRequest.source(searchSourceBuilder);
        SearchResponse searchResponse = getClient.search(searchRequest);
        List<String> collect = Arrays.stream(searchResponse.getHits().getHits())
                .map(x -> {
                    Map<String, Object> sourceAsMap = x.getSourceAsMap();
                    String str = JSONObject.toJSONString(sourceAsMap);
                    JSONObject c = JSONObject.parseObject(str);
                    String show_name = c.getJSONObject("cname").getString("show");
                    return show_name;
                }).collect(Collectors.toList());
        if (collect.isEmpty()) return "";
        return collect.get(0);
    }
}
