package com.winhc.task;

import cn.hutool.core.lang.Tuple;
import com.alibaba.hologres.client.HoloClient;
import com.alibaba.hologres.client.exception.HoloClientException;
import com.aliyun.odps.Column;
import com.aliyun.odps.Odps;
import com.aliyun.odps.Table;
import com.aliyun.odps.TableSchema;
import com.winhc.task.bean.JobArgs;
import com.winhc.task.common.Constant;
import com.winhc.task.common.SummaryArgs;
import com.winhc.task.util.FreeMarkUtil;
import com.winhc.task.util.HoloUtils;
import com.winhc.task.util.OdpsUtils;
import lombok.SneakyThrows;
import org.apache.calcite.avatica.proto.Common;
import org.apache.commons.lang3.StringUtils;
import org.junit.jupiter.api.Test;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.boot.test.context.SpringBootTest;

import java.io.IOException;
import java.util.*;
import java.util.stream.Collectors;
import java.util.stream.Stream;

@SpringBootTest
public class SJTaskJob {
    @Autowired
    FreeMarkUtil freeMarkUtil;

    @Test
    public void start() throws IOException {
        Map<String, String> m = new HashMap<>();
        m.put("targetTableName", "test_tmp_xf_sum_v9");
        m.put("tn", "zxr_evaluate_results");
        m.put("keyno", "keyno");
        m.put("groupField", "deleted");
        m.put("condition", "in ('0','1')");
        String s = freeMarkUtil.genStr("agg_sample.ftl", m);
        System.out.println(s);
    }

    @Test
    public void start2() throws IOException {
        List<String> list = Arrays.asList("bankruptcy_judgment_document", "bankruptcy_open_announcement", "bankruptcy_open_case"
                , "company_court_open_announcement", /*"company_court_announcement",*/ "company_send_announcement"
                , "company_court_register", "litigation_mediation", "restrictions_on_exit", "auction_tracking"/*,"property_rights_transaction"*/);
        list.forEach(tn -> {
            Map<String, String> m = new HashMap<>();
            m.put("tn", tn);
            String s = freeMarkUtil.genStr("truncate_table.ftl", m);
            System.out.println(s);
        });
    }

    @Test
    public void start3() throws IOException {
        List<String> list = Arrays.asList("bankruptcy_judgment_document", "bankruptcy_open_announcement", "bankruptcy_open_case"
                , "company_court_open_announcement", /*"company_court_announcement",*/ "company_send_announcement"
                , "company_court_register", "litigation_mediation", "restrictions_on_exit", "auction_tracking"/*,"property_rights_transaction"*/);
        list.forEach(tn -> {
            Map<String, String> m = new HashMap<>();
            m.put("tn", tn);
            String s = freeMarkUtil.genStr("test1.ftl", m);
            System.out.println(s);
        });
    }

    public static List<String> tns = Arrays.asList(/*"company",*/
            "company_holder",
            "company_staff",
            "company_change",
            "company_dishonest_info",
            "company_zxr",
            "company_zxr_restrict",
            "company_zxr_final_case",
            "company_send_announcement",
            "wenshu_detail_v2",
            "company_court_open_announcement",
            "company_court_register",
            "company_court_announcement",
            "company_judicial_assistance",
            "company_punishment_info",
            "company_abnormal_info",
            "company_equity_info",
            "company_mortgage_info",
            "company_mortgage_pawn",
            "company_mortgage_change",
            "company_mortgage_people",
            "company_env_punishment",
            "company_tax_contravention",
            "company_own_tax",
            "company_illegal_info",
            "company_equity_pledge",
            "company_ipr_pledge",
            "company_certificate",
            "company_license",
            "company_tax");

    @Test
    public void calcOdpsMerge() throws IOException {
        tns.forEach(tn -> {
            Map<String, String> m = new HashMap<>();
            m.put("tn", tn);
            String s = freeMarkUtil.genStr("merge_sql.ftl", m);
            System.out.println(s);
        });
    }

    @SneakyThrows
    @Test
    public void calcHolo() throws Exception {
        Map<String, SummaryArgs> a = SummaryArgs.SUMMARY_ARGS;
        List<String> pid_tns = Arrays.asList("company_dishonest_info",
                "company_zxr",
                "company_zxr_restrict",
                "company_zxr_final_case");
        List<Tuple> sql_list = tns.stream().filter(a::containsKey).map(tn -> {
            SummaryArgs args = a.get(tn);
            String p_sql = args.getArgsInfo().stream().map(x -> {
                Map<String, String> m = new HashMap<>();
                m.put("tn", tn);
                m.put("keyno", x.getFilterField());
                return freeMarkUtil.genStr("sj_mapping_v1.ftl2", m);
            }).collect(Collectors.joining("\nUNION ALL\n"));

            Map<String, String> m = new HashMap<>();
            m.put("table_view", p_sql);
            m.put("human_pid_view", "");
            if (pid_tns.contains(tn)) {
                m.put("human_pid_view", "UNION ALL\n" +
                        "            SELECT  human_pid as id\n" +
                        "            FROM    tmp_xf_human_pid_sj_view");
            }
            m.put("tn", tn);
            String calc_sql = freeMarkUtil.genStr("sj_mapping_v2.ftl2", m);
            return new Tuple(tn, calc_sql);
        }).collect(Collectors.toList());

        calcHolo(sql_list);
    }

    private void calcHolo(List<Tuple> collect) throws HoloClientException {
        String target_table = "tmp_xf_all_rowkey_tn";
        for (int i = 0; i < collect.size(); i++) {
            HoloClient holoClient = HoloUtils.init();
            String s0 = "";
            //初始化清空table
            if (i == 0) {
                s0 = "truncate table " + target_table;
            }
            String tn = collect.get(i).get(0).toString();
            String s1 = "analyze " + target_table;
            String s2 = "analyze ng_rt_" + tn;
            Stream.of(s0, s1, s2, collect.get(i).get(1)).filter(StringUtils::isNotBlank).forEach(x -> HoloUtils.exexSql(holoClient, x));
            holoClient.close();
        }
    }

    @Test
    public void calcOdpsExport() throws IOException {
        List<String> subTns = Arrays.asList("company_mortgage_pawn", "company_mortgage_change", "company_mortgage_people");
        tns.forEach(tn -> {
            Map<String, String> m = new HashMap<>();
            m.put("tn", tn);
            String cols = getCols(tn);
            m.put("cols", cols);
            String s;
            if (subTns.contains(tn)) {
                s = freeMarkUtil.genStr("merge_sql_mapping_v2.ftl", m);
            } else {
                s = freeMarkUtil.genStr("merge_sql_mapping_v1.ftl", m);
            }
            System.out.println(s);
        });
    }

    // 获取表结构
    public static String getCols(String tn) {
        String tableName = "ads_" + tn + "_v9";
        // 初始化odps
        Odps odps = OdpsUtils.getOdps();
        // 获取表
        Table table = odps.tables().get(tableName);

        // 获取表结构
        TableSchema tableSchema = table.getSchema();

        //获取列名列注释
        List<Column> columns = tableSchema.getColumns();

        List<String> cols = new ArrayList<>();

        //存在过滤字段跳过
        List<String> filter_cols = Constant.tn_cols_filters.getOrDefault(tn, new ArrayList<>());
        //列字段
        for (Column column : columns) {
            // 判断如果ds存在，则跳过
            if (filter_cols.contains(column.getName()) || "ds".equalsIgnoreCase(column.getName())) {
                continue;
            }

            cols.add(column.getName());
        }

        return cols.stream().collect(Collectors.joining(",","a.",""));
    }
}